%define anolis_release 1
%define asciidoctor -Dasciidoctor=enabled
%define libtracefs -Dlibtracefs=enabled

Name:		ndctl
Version:	80
Release:	%{anolis_release}%{?dist}
Summary:	Manage "libnvdimm" subsystem devices (Non-volatile Memory)
License:	GPLv2
Url:		https://github.com/pmem/ndctl
Source0:	https://github.com/pmem/%{name}/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz

Requires:	ndctl-libs%{?_isa} = %{version}-%{release}
Requires:	daxctl-libs%{?_isa} = %{version}-%{release}
Requires:	cxl-libs%{?_isa} = %{version}-%{release}
BuildRequires:	autoconf automake libtool meson
BuildRequires:	rubygem-asciidoctor
BuildRequires:	libtraceevent-devel
BuildRequires:	libtracefs-devel
BuildRequires:	xmlto
BuildRequires:	pkgconfig
BuildRequires:	pkgconfig(libkmod)
BuildRequires:	pkgconfig(libudev)
BuildRequires:	pkgconfig(uuid)
BuildRequires:	pkgconfig(json-c)
BuildRequires:	pkgconfig(bash-completion)
BuildRequires:	pkgconfig(systemd)
BuildRequires:	keyutils-libs-devel
BuildRequires:	systemd-rpm-macros
BuildRequires:	iniparser-devel

%description
NDCTL (Non-volatile DIMM Control) is a set of tools and utilities
that provide a command-line interface for managing non-volatile
memory (NVM) devices or NVDIMMs on Linux systems. NVDIMMs are a
type of memory that combines the speed of DRAM with the persistence
of storage. They are essentially DIMMs (Dual In-line Memory Modules)
with an additional NVM component that enables data persistence
even in the event of power failure.

NDCTL is designed to provide low-level access to the capabilities
of NVDIMMs and to make the management and configuration of these
devices easier. It is a user-space utility and does not require
kernel modifications. The tools and utilities included in NDCTL
can be used to create, delete, and resize namespaces, configure
memory modes, and monitor the health and status of NVDIMMs.

Namespaces are logical partitions of an NVDIMM that can be used
to store data. NDCTL allows users to create and manage namespaces
on NVDIMMs. Namespaces can be created in various modes, such as raw,
block, or filesystem. The raw mode provides direct access to the
NVDIMM memory, while block mode presents the NVDIMM as a block device
to the operating system. Filesystem mode, on the other hand, provides
a filesystem interface to the NVDIMM.

Memory mode is another important aspect of NVDIMM management.
NDCTL provides tools to configure memory modes such as "memory"
mode or "block" mode. In memory mode, the NVDIMM appears as a
volatile memory that can be used as system memory, while in block
mode, it appears as a block device that can be used for storage.

NDCTL also provides a health monitoring system that can detect
any issues with NVDIMMs. The tools can be used to monitor the
health of NVDIMMs, detect any errors such as uncorrectable memory
errors, and provide early warning of impending failures. This
can help system administrators to take corrective actions before
a failure occurs.

NDCTL is an open-source project that is actively maintained
and supported by the Linux community. It supports various
NVDIMM types such as NVDIMM-N, NVDIMM-F, and NVDIMM-P. NDCTL
is a powerful tool for managing and configuring NVDIMMs on
Linux systems and is widely used in data centers and other
environments that require high-performance data persistence.

%package -n ndctl-devel
Summary:	Development files for libndctl
License:	LGPLv2
Requires:	ndctl-libs%{?_isa} = %{version}-%{release}

%description -n ndctl-devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%package -n daxctl
Summary:	Manage Device-DAX instances
License:	GPLv2
Requires:	daxctl-libs%{?_isa} = %{version}-%{release}

%description -n daxctl
The daxctl utility provides enumeration and provisioning commands for
the Linux kernel Device-DAX facility. This facility enables DAX mappings
of performance / feature differentiated memory without need of a
filesystem.

%package -n cxl-cli
Summary:	Manage CXL devices
License:	GPLv2
Requires:	cxl-libs%{?_isa} = %{version}-%{release}
Requires:	daxctl-libs = %{version}-%{release}

%description -n cxl-cli
The cxl utility provides enumeration and provisioning commands for
the Linux kernel CXL devices.

%package -n cxl-devel
Summary:	Development files for libcxl
License:	LGPLv2
Requires:	cxl-libs%{?_isa} = %{version}-%{release}

%description -n cxl-devel
This package contains libraries and header files for developing applications
that use libcxl, a library for enumerating and communicating with CXL devices.

%package -n daxctl-devel
Summary:	Development files for libdaxctl
License:	LGPLv2
Requires:	daxctl-libs%{?_isa} = %{version}-%{release}

%description -n daxctl-devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}, a library for enumerating
"Device DAX" devices.  Device DAX is a facility for establishing DAX
mappings of performance / feature-differentiated memory.


%package -n ndctl-libs
Summary:	Management library for "libnvdimm" subsystem devices (Non-volatile Memory)
License:	LGPLv2
Requires:	daxctl-libs = %{version}-%{release}

%description -n ndctl-libs
Libraries for %{name}.

%package -n daxctl-libs
Summary:	Management library for "Device DAX" devices
License:	LGPLv2

%description -n daxctl-libs
Device DAX is a facility for establishing DAX mappings of performance /
feature-differentiated memory. daxctl-libs provides an enumeration /
control API for these devices.

%package -n cxl-libs
Summary:	Management library for CXL devices
License:	LGPLv2
Requires:	daxctl-libs = %{version}-%{release}

%description -n cxl-libs
libcxl is a library for enumerating and communicating with CXL devices.

%package doc
Summary: Documentation files for %{name}
Requires: %{name} = %{version}-%{release}
BuildArch: noarch

%description doc
The %{name}-doc package contains documentation files for %{name}

%prep
%setup -q ndctl-%{version}

%build
%meson %{?asciidoctor} %{?libtracefs} -Dversion-tag=%{version}
%meson_build

%install
%meson_install

%generate_compatibility_deps

%check
%meson_test

%ldconfig_scriptlets -n ndctl-libs

%ldconfig_scriptlets -n daxctl-libs

%ldconfig_scriptlets -n cxl-libs

%define bashcompdir %(pkg-config --variable=completionsdir bash-completion)

%pre
if [ -f %{_sysconfdir}/ndctl/monitor.conf ] ; then
  if ! [ -f %{_sysconfdir}/ndctl.conf.d/monitor.conf ] ; then
    cp -a %{_sysconfdir}/ndctl/monitor.conf /var/run/ndctl-monitor.conf-migration
  fi
fi

%post
if [ -f /var/run/ndctl-monitor.conf-migration ] ; then
  config_found=false
  while read line ; do
    [ -n "$line" ] || continue
    case "$line" in
      \#*) continue ;;
    esac
    config_found=true
    break
  done < /var/run/ndctl-monitor.conf-migration
  if $config_found ; then
    echo "[monitor]" > %{_sysconfdir}/ndctl.conf.d/monitor.conf
    cat /var/run/ndctl-monitor.conf-migration >> %{_sysconfdir}/ndctl.conf.d/monitor.conf
  fi
  rm /var/run/ndctl-monitor.conf-migration
fi

%files
%dir %{abidir}
%license COPYING
%license LICENSES/preferred/GPL-2.0 LICENSES/other/MIT LICENSES/other/CC0-1.0
%{_bindir}/ndctl
%{abidir}/ndctl-option.list
%{_mandir}/man1/ndctl*
%{bashcompdir}/ndctl
%{_unitdir}/ndctl-monitor.service
%dir %{_sysconfdir}/ndctl
%dir %{_sysconfdir}/ndctl/keys
%{_sysconfdir}/ndctl/keys/keys.readme
%{_sysconfdir}/modprobe.d/nvdimm-security.conf

%dir %{_sysconfdir}/ndctl.conf.d
%config(noreplace) %{_sysconfdir}/ndctl.conf.d/monitor.conf
%config(noreplace) %{_sysconfdir}/ndctl.conf.d/ndctl.conf

%files -n daxctl
%license LICENSES/preferred/GPL-2.0 LICENSES/other/MIT LICENSES/other/CC0-1.0
%license COPYING
%{_bindir}/daxctl
%{abidir}/daxctl-option.list
%{_mandir}/man1/daxctl*
%{_datadir}/daxctl/
%{bashcompdir}/daxctl
%{_unitdir}/daxdev-reconfigure@.service
%config %{_udevrulesdir}/90-daxctl-device.rules
%dir %{_sysconfdir}/daxctl.conf.d/
%config(noreplace) %{_sysconfdir}/daxctl.conf.d/daxctl.example.conf

%files -n cxl-cli
%license LICENSES/preferred/GPL-2.0 LICENSES/other/MIT LICENSES/other/CC0-1.0
%license COPYING
%{_bindir}/cxl
%{_mandir}/man1/cxl*
%{bashcompdir}/cxl
%{_unitdir}/cxl-monitor.service

%files -n ndctl-libs
%doc README.md
%license COPYING
%license LICENSES/preferred/LGPL-2.1 LICENSES/other/MIT LICENSES/other/CC0-1.0
%{_libdir}/libndctl.so.*
%{abidir}/libndctl.dump

%files -n daxctl-libs
%doc README.md
%license COPYING
%license LICENSES/preferred/LGPL-2.1 LICENSES/other/MIT LICENSES/other/CC0-1.0
%{_libdir}/libdaxctl.so.*
%{abidir}/libdaxctl.dump
%{abidir}/daxctl-option.list

%files -n cxl-libs
%doc README.md
%license LICENSES/preferred/LGPL-2.1 LICENSES/other/MIT LICENSES/other/CC0-1.0
%license COPYING
%{_libdir}/libcxl.so.*
%{abidir}/libcxl.dump
%{abidir}/cxl-option.list

%files -n ndctl-devel
%license LICENSES/preferred/LGPL-2.1
%license COPYING
%{_includedir}/ndctl/
%{_libdir}/libndctl.so
%{_libdir}/pkgconfig/libndctl.pc

%files -n daxctl-devel
%license LICENSES/preferred/LGPL-2.1
%license COPYING
%{_includedir}/daxctl/
%{_libdir}/libdaxctl.so
%{_libdir}/pkgconfig/libdaxctl.pc

%files -n cxl-devel
%license LICENSES/preferred/LGPL-2.1
%license COPYING
%{_includedir}/cxl/
%{_libdir}/libcxl.so
%{_libdir}/pkgconfig/libcxl.pc
%{_mandir}/man3/cxl*
%{_mandir}/man3/libcxl.3*

%files doc
%doc CONTRIBUTING.md README.md

%changelog
* Wed Mar 26 2025 Hong Wei Qin <qhw01063182@alibaba-inc.com> - 80-1
- Update to 80-1 from 73-2

* Mon Apr 10 2023 Xuchun Shang <xuchun.shang@linux.alibaba.com> - 73-2
- Add doc pkg and abi file

* Mon Apr 18 2022 Yuanhong Peng <yummypeng@linux.alibaba.com> - 73-1
- Init package from upstream v73
